#!/usr/bin/python

#
# Phonebook: a simple script to access an address book / phone book,
# compatible with the "pilot-address" program from pilot-link.
#
# Version: 2002-11-05
#
# A palm pilot, and/or pilot-link, is suggested but not required.
#
# Copyright (c) 2002 Scott Scriven
# This software is freely redistributable under the terms of the 
# GNU Public License.  Read the file "COPYING" for full details.
#
# You may want to edit the names of the four custom fields, since
# these are not included in the address book file.  They are set to
# what I use.
#

# TODO:
# - save data to file
# - add entries
# - remove entries
# - edit entries

class phonebook:
  def __init__(self, cfg=None):
    self.fields = [ 
       "Last name",
       "First name",
       "Title",
       "Company",
       "Work",
       "Home",
       "Fax",
       "Other",
       "email",
       "Address",
       "City",
       "State",
       "Zip",
       "Country",
       "Alias",    # custom 1
       "From",     # custom 2
       "Birthday", # custom 3
       "???",      # custom 4
       "Note",
       #"0",	   # (zero) 
       ]
    self.raw_entries = []

  def load(self, filename):
    try:
      fp = open(filename, "rb")
      lines = fp.readlines()
      fp.close()
    except Exception, msg:
      print "Error loading phonebook: %s" % (msg)

    self.raw_entries = self._load_entries(lines)

  def list(self, patterns=[]):
    if not patterns:
      return self.list_all()

    for e in self.raw_entries:
      import string

      found=0
      for p in patterns:
	if 0 <= string.find(string.lower(e), string.lower(p)):
	  found=1

      if found:
	person = self._parse_entry(e)
	self.show_entry(person)
	print ""

  def list_all(self):
    for e in self.raw_entries:
      person = self._parse_entry(e)
      self.show_entry(person)
      print ""

  def _load_entries(self, lines):
    "create a list of unparsed entries..."
    unterminated = 0
    entries = []
    for line in lines:
      line = line[:-1]
      #print "Line: %s" % (line)
      if unterminated:
	if line[-3:] == '"0"':
	  unterminated = 0
	text = entries[-1]
	text = text + line + "\n"
	entries[-1] = text
      elif line[-3:] != '"0"':
	unterminated = 1
	entries.append(line)
      else:
	unterminated = 0
	entries.append(line)

    return entries

  def _parse_entry(self, e):
    import string
    e = e[1:-1]
    eparts = string.split(e, '","')
    return eparts

  def show_entry(self, e):
    import string

    format = "%s: %s"
    start = 0
    if e[0] or e[1]:
      print "%s %s" % (e[1], e[0])
      format = "%14s: %s"
      start = 2

    for f in range(start, len(self.fields)):
      if e[f]:
	value = e[f]
	if "\n" in value:
	  value = string.replace(value, "\n", "\n\t\t")
	print format % (self.fields[f], value)
	format = "%14s: %s"

#### Now do..  *something* with the phone book.
#
#if($command eq "find"  ||  $command eq "list") {
#
#  if($ARGV[0] eq "") {
#    foreach $item (@list)  {
#	&show($item);
#    }
#    exit(0);
#  }
#    
#  foreach $item (@list) 
#  {
#      foreach $word (@ARGV) 
#      {
#	  if($item =~ /$word/i) 
#	  {
#	      &show($item);
#	  }
#      }
#  }
#  exit(0);
#}
#
#if($command eq "add") {
#
#    $item = "";
#
#    for($i=0; $fields[$i] =~ /\S/; $i++)
#    {
#	print "$fields[$i]: ";
#	$data[$i] = <>;
#	chop $data[$i];
#	$item .= "\"$data[$i]\",";
#    }
#
#    $item .= "\"0\"\n";
#
#    print "You have added...\n";
#    &show($item);
#
#  
#    push(@list, $item);
#
#    &write();
#  
#    exit(0);
#}
#
#if($command eq "remove"  ||  $command eq "del") 
#{
#  for ($j=0; $j <= $#list; $j++) {
#    $item = $list[$j];
#    @data = split(/\|/, $item);
#    foreach $word (@ARGV) {
#      if($item =~ /$word/i) {
#	  &show($item);
#	  print "Remove this entry? [y/N] ";
#	  $tmp = <STDIN>; chop $tmp;
#	  if ($tmp =~ /^y/i) {splice(@list, $j--, 1);}
#      }
#    }
#  }
#
#  &write();
#
#  exit(0);
#}
#
#

def run(args):
  if len(args) < 1:
    print "Simple Phonebook Program - Usage:"
    print "\tphone [names]       \tTo look up names/numbers..."
    print "\tphone list [names]  \tTo look up names/numbers..."
    #print "\tphone find [names]  \t... (same thing) ..."
    #print "\tphone add           \tAdd new entries"
    #print "\tphone remove [names]\tRemove all entries which match [names]"
    #print "\tphone del [names]   \t... (same thing) ..."
    return

  else:
    import os
    try:
      home = os.environ["HOME"]
    except:
      home = "/tmp"
    phonebook_file = home + "/.phonebook";

    command = args[0]
    patterns = args[1:]
    pb = phonebook()
    pb.load(phonebook_file)

    if command in ["find", "list"]:
      pb.list(patterns)
    else:
      pb.list(args)

if __name__ == "__main__":
  import sys
  run(sys.argv[1:])
